extension StringExtensions on String? {
  bool isNullOrEmpty() => this?.isEmpty ?? true;

  String emptyIfNull() => this ?? "";

  String defaultIfEmpty(String defaultVal) => isNullOrEmpty() ? defaultVal.emptyIfNull() : this!;

  String? capitalize() => isNullOrEmpty() ? this : '${this![0].toUpperCase()}${this!.substring(1)}';

  bool equalsIgnoreCase(String? lookup) {
    return this?.toLowerCase() == lookup?.toLowerCase();
  }

  bool containsIgnoreCase(String? lookup) {
    return lookup != null && this!.toLowerCase().contains(lookup.toLowerCase());
  }

  String truncateWithEllipsis(int maxLength) {
    if (this == null || this!.length <= maxLength) return this ?? '';
    return '${this!.substring(0, maxLength - 3)}...';
  }

  /// Returns: <br>
  /// [empty] if the value is null,
  /// [current value] if only a single word
  /// [formal name]
  /// <br><br>
  /// eg: [null.toFormalName() => ' ']<br>
  /// eg: ['stubbly'.toFormalName() => 'stubbly']<br>
  /// eg: ['Matthew Dodds'.toFormalName() => 'M Dodds']<br>
  /// <br><br>
  // Regex Breakdown:
  //   '^(\w)\w* ' - take the first char of the first word from the start of the string
  //   '(.*)$' - take the remaining value from the string after the first word and ' '
  //   If the string does not match no replacement is performed.
  String toFormalName() => isNullOrEmpty()
      ? ''
      : this!.replaceFirstMapped(RegExp(r'^(\w)\w* (.*)$'), (m) => '${m[1]} ${m[2] ?? ''}');
}